/*
 * Copyright (c) 1999 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * tonetio.{cc,hh} -- Send packets to a OSKit netio.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "tonetio.hh"
#include "error.hh"
#include "packet.hh"
#include "confparse.hh"
#include "glue.hh"
#include <unistd.h>
#include "netio_guts.h"

ToNetIO::ToNetIO()
  : Element(1, 0), _ifnum(-1)
{
}

ToNetIO::~ToNetIO()
{
	if (_ifnum >= 0)
		(void) oskit_destroy_tonetio(_ifnum, _priv);
}

ToNetIO *
ToNetIO::clone() const
{
	return new ToNetIO;
}

int
ToNetIO::configure(const String &conf, ErrorHandler *errh)
{
	if (cp_va_parse(conf, this, errh,
			cpInteger, "interface number", &_ifnum,
			cpEnd) < 0)
		return -1;

	return 0;
}

int
ToNetIO::initialize(ErrorHandler *)
{
	return oskit_create_tonetio(this, _ifnum, &_priv);
}

void
ToNetIO::push(int, Packet *p)
{
	/*
	 * Kludgey. 
	 */
	p->bufio_convert();
	oskit_sendto_netio_withbufio(_priv, p->bufio(), p->length());
	p->kill();
}

bool
ToNetIO::wants_packet_upstream() const
{
  return input_is_pull(0);
}

void
ToNetIO::run_scheduled()
{
  while (Packet *p = input(0).pull())
    push(0, p);
}



EXPORT_ELEMENT(ToNetIO)
