/*
 * Copyright (c) 1999 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Click COM component.
 */

#ifndef _CLICKER_H_
#define _CLICKER_H_

#include <oskit/types.h>
#include <oskit/com.h>

struct oskit_clicker {
	struct oskit_clicker_ops *ops;
};
typedef struct oskit_clicker oskit_clicker_t;

struct oskit_clicker_ops {
	/* COM-specified IUnknown interface operations. */
	OSKIT_COMDECL_IUNKNOWN(oskit_clicker_t)

	/*** Operations specific to the oskit_clicker interface ***/

	/*
	 * Configure a Click component using a configuration file.
	 */
	OSKIT_COMDECL	(*config)(oskit_clicker_t *c, char *config_file);

	/*
	 * Start the Click driver up.
	 */
	OSKIT_COMDECL	(*start)(oskit_clicker_t *c);

	/*
	 * Make it stop.
	 */
	OSKIT_COMDECL_V	(*stop)(oskit_clicker_t *c);
};

extern const struct oskit_guid oskit_clicker_iid;
#define OSKIT_CLICKER_IID OSKIT_GUID(0x4aa7dff8, 0x7c74, 0x11cf, \
                0xb5, 0x00, 0x08, 0x00, 0x09, 0x53, 0xad, 0xc2)

#define oskit_clicker_query(c, iid, out_ihandle) \
	((c)->ops->query((oskit_clicker_t *)(c), (iid), (out_ihandle)))
#define oskit_clicker_addref(c) \
	((c)->ops->addref((oskit_clicker_t *)(c)))
#define oskit_clicker_release(c) \
	((c)->ops->release((oskit_clicker_t *)(c)))

#define oskit_clicker_config(c, file) \
	((c)->ops->config((oskit_clicker_t *)(c), (file)))
#define oskit_clicker_start(c) \
	((c)->ops->start((oskit_clicker_t *)(c)))
#define oskit_clicker_stop(c) \
	((c)->ops->stop((oskit_clicker_t *)(c)))

/*
 * Create an instance of a click object. The object is pretty much useless
 * until it is configured, and then started.
 */
oskit_clicker_t *oskit_clicker_create(void);

#endif /* _CLICKCOM_H_ */
